###------------Setup and Understanding Data Types-----------------------------
# 
# By Danielle Walkup (with lots of help from Google) 
# EEB OSOS Workshop, August 2019
#
# This script goes with the OSOS-2019 handout, section 3.1 
# 
###------------Packages and working directory-----------------------------------
#
## One of the earlier parts of your script should include a list of packages that
## will be used in the script. I generally set it up as a commented out install.packages()
## command (because I typically have them already loaded), along with a set of
## library(), that loads each package
#install.packages("tidyverse)
library(tidyverse)
#getwd()
#setwd()
#
###------------Types of data-------------------------------------------------
## Some examples:
?class()  #we can learn a little something about the classes in R
?typeof()
#
# The classes can vary depending on the type of data:
liz.wt.num <- c(1.4, 5.6, 8.3, 5.6, 5.0)
class(liz.wt.num)  #returns numbers
typeof(liz.wt.num)

liz.wt.ch <- c("1.4", "5.6", "8.3", "5.6", "5.0")
class(liz.wt.ch)  #returns characters
typeof(liz.wt.ch)

# if you need to change the class, you can:
liz.wt.change <- as.numeric(liz.wt.ch)
class(liz.wt.change)

liz.wt.change2 <- as.character(liz.wt.num)
class(liz.wt.change2)

# Notice what happens when we change them to integers:
liz.wt.int <- as.integer(liz.wt.ch)
class(liz.wt.int)
typeof(liz.wt.int)
liz.wt.int

## Factors
# Ordering of factors defaults to alphabetical:
fruit <- factor(c("apple","pear","banana","grape"))
fruit
# But we can change that if we want to:
fruit2 <- factor(c("apple","pear","banana","grape")
                  , levels = c("apple","pear","banana","grape"))
fruit2
#
# We can also create an ordinal variable:
fruit.ord <- factor(c("apple",'pear',"banana","grape"), ordered = TRUE)
fruit.ord  #note the < between the levels now AND our levels changed back to alphabetical
fruit.ord2 <- factor(c("apple",'pear',"banana","grape")
                     , levels = c("apple","pear","banana","grape")
                     , ordered = TRUE)
fruit.ord2  #Much better - if you like grapes...
#
# Notice there are some constraints on classes:
fruit.ch <- as.character(fruit)
class(fruit.ch)
fruit.ch

fruit.num <- as.numeric(fruit.ch)  #It'll do it, but it'll do it badly.
class(fruit.num)
fruit.num
#
# BUT, be careful, because we can coerce factors to numbers or integers:
fruit.num2 <- as.numeric(fruit)  #using the alphabetically ordered fruit
class(fruit.num2)
fruit.num2  #Note that it keeps the ordering

fruit.int <- as.integer(fruit2)  #using our fruit orders
class(fruit.int)
fruit.int  #Again, note that it keeps the changed ordering
# 
###-------------Data Objects Examples-------------------------------------------
#
## Vectors - collection of elements of the same type
a <- c(1, 2, 3, 4, 5)
b <- c("apple", "pear", "banana", "grape")

# We can see elements in the vectors:
b[3] #the single bracket lets us see the element in the 3rd place in the vector
b[8] #there's nothing in here - because we only have the 4 fruits!

# We can see all sorts of inforamtion about the vectors:
class(a)  #notice this returns the atomic vector type
length(b)
is.na(a)  #a handy function to check for missing values
sum(a)  #we can run functions specific to the given class
sum(b)  #error because we have no numbers to add!
str(a)

## Lists
ex_list <- list(1, "a", TRUE, 1+4i)
class(ex_list)  #now we see this is a list, with its special list properties!
# We can still see the different elements and characteristics
ex_list[2]
length(ex_list)
str(ex_list)
# You can also name the elements in each list
ex_biglist <- list(a = "Testing", b = 1:10, flowers = head(iris))
#
# now if we want to see what's in the list:
names(ex_biglist)  #we can also see the names
ex_biglist[2]  #and what's in the list
ex_biglist$b  #alternatively, if we have elements in the list named, we can use the
            # '$' to choose the different names and show what's in them
length(ex_biglist)  #but we still only have 3 things in the list

## Matrices
ex_mat <- matrix(1:10, nrow = 2)
ex_mat
ex_mat2 <- matrix(1:10, nrow = 2, byrow = TRUE)   
#TIP: instead of writing out 1,2,3,4,5,6,7,8,9,10, we use 1:10 to give us the 
#     sequence of numbers
ex_mat2

# and we can see characteristics:
class(ex_mat)
str(ex_mat)
length(ex_mat)
dim(ex_mat)  #Now we can see the dimensions!
ex_mat[2,1]  #now that we have 2 dimensions, we can use [row,column] to id an element
ex_mat[,2]  #or see just a column
ex_mat[2,]  #or see just a row
sum(ex_mat)  #can still do math on a numeric matrix - 
t(ex_mat)  #can also transpose a matrix 

## Data frames
# here we will create a data frame with 3 columns: id, height, and width
ex_df <- data.frame(id = rep(c("a","b","c"),4), height = 1:12, width = 12:1)
ex_df  #lets make sure it did what we expected

# and check out some of its characteristics:
class(ex_df)
str(ex_df)
summary(ex_df)
length(ex_df)  #we can still get a length, but it may not quite be the info we want
dim(ex_df)  
t(ex_df)  #we can still transpose the data frame

# and we can see the different parts:
ex_df$id  #note that id has levels, so it is a factor
class(ex_df$id)  #which we can double check here
class(ex_df$width)  #and it has defaulted width to integer, which we could change:
as.numeric(ex_df$width)
class(ex_df$width)  #wait, I though we changed this?!
# with data frames we need to re-assign the column to "save" the changes
ex_df$width <- as.numeric(ex_df$width)
class(ex_df$width)
#
# We could even do math using columns from the data frame:
sum(ex_df$width)
mean(ex_df$width)

## Subsetting our data frame
# we've subsetted throughout this script using brackets [], but there are lots 
# of ways to subset our database:
ex_df[1, ] #just a row
ex_df[, 1] #just a column
ex_df[2, 2] #row 2 and column 2

# we can subset based on different criteria
ex_df[which(ex_df$id == "a"), ] 
ex_df[ex_df$id %in% "a", ] 
subset(ex_df, id == "a")
ex_df %>% subset(id == "a")

# or different combinations of criteria
ex_df %>% subset(height >= 8)
ex_df %>% subset(height >= 9 & width <= 2)
ex_df %>% subset(height >= 10 | width >= 10)
ex_df %>% subset(height >= 10 & width <= 2) %>% select(id)
#